from __future__ import print_function

import pickle
import gevent.testing as greentest
try:
    from gevent.resolver.cares import ares_host_result
except ImportError: # pragma: no cover
    ares_host_result = None


@greentest.skipIf(ares_host_result is None,
                  "Must be able to import ares")
class TestPickle(greentest.TestCase):
    # Issue 104: ares.ares_host_result unpickleable

    def _test(self, protocol):
        r = ares_host_result('family', ('arg1', 'arg2', ))
        dumped = pickle.dumps(r, protocol)
        loaded = pickle.loads(dumped)
        self.assertEqual(r, loaded)
        self.assertEqual(r.family, loaded.family)


for i in range(0, pickle.HIGHEST_PROTOCOL):
    def make_test(j):
        return lambda self: self._test(j)
    setattr(TestPickle, 'test' + str(i), make_test(i))


if __name__ == '__main__':
    greentest.main()



import gevent

from gevent import testing as greentest


class Test(greentest.TestCase):

    def test(self):

        def func():
            pass


        a = gevent.spawn(func)
        b = gevent.spawn(func)
        gevent.joinall([a, b, a])


if __name__ == '__main__':
    greentest.main()



# -*- coding: utf-8 -*-
"""
Tests that on Python 2, if the futures backport of 'thread' is already
imported before we monkey-patch, it gets patched too.
"""

import unittest

try:
    import thread
    import _thread
    HAS_BOTH = True
except ImportError:
    HAS_BOTH = False

class TestMonkey(unittest.TestCase):

    @unittest.skipUnless(HAS_BOTH, "Python 2, needs future backport installed")
    def test_patches_both(self):
        thread_lt = thread.LockType
        _thread_lt = _thread.LockType
        self.assertIs(thread_lt, _thread_lt)

        from gevent.thread import LockType as gLockType

        self.assertIsNot(thread_lt, gLockType)

        import gevent.monkey
        gevent.monkey.patch_all()

        thread_lt2 = thread.LockType
        _thread_lt2 = _thread.LockType

        self.assertIs(thread_lt2, gLockType)
        self.assertIs(_thread_lt2, gLockType)

        self.assertIs(thread_lt2, _thread_lt2)
        self.assertIsNot(thread_lt2, thread_lt)

        # Retrieving the original on the old name still works
        orig_locktype = gevent.monkey.get_original('thread', 'LockType')
        self.assertIs(orig_locktype, thread_lt)

        # And the new name
        orig__locktype = gevent.monkey.get_original('_thread', 'LockType')
        self.assertIs(orig__locktype, thread_lt)

if __name__ == '__main__':
    unittest.main()



from __future__ import print_function
import gevent
import gevent.core
import time
try:
    import thread
except ImportError:
    import _thread as thread

from gevent import testing as greentest

class Test(greentest.TestCase):
    def test(self):
        hub = gevent.get_hub()
        watcher = hub.loop.async_()

        # BWC for <3.7: This should still be an attribute
        assert hasattr(hub.loop, 'async')

        gevent.spawn_later(0.1, thread.start_new_thread, watcher.send, ())

        start = time.time()

        with gevent.Timeout(1.0): # Large timeout for appveyor
            hub.wait(watcher)

        print('Watcher %r reacted after %.6f seconds' % (watcher, time.time() - start - 0.1))


if __name__ == '__main__':
    greentest.main()



# Copyright (c) 2008 AG Projects
# Author: Denis Bilenko
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.

import sys
import gevent.testing as greentest
import weakref
import time
import gc

from gevent import sleep
from gevent import Timeout
from gevent import get_hub


from gevent.testing.timing import SMALL_TICK as DELAY
from gevent.testing import flaky


class Error(Exception):
    pass


class _UpdateNowProxy(object):

    update_now_calls = 0

    def __init__(self, loop):
        self.loop = loop

    def __getattr__(self, name):
        return getattr(self.loop, name)

    def update_now(self):
        self.update_now_calls += 1
        self.loop.update_now()

class _UpdateNowWithTimerProxy(_UpdateNowProxy):

    def timer(self, *_args, **_kwargs):
        return _Timer(self)

class _Timer(object):

    pending = False
    active = False

    def __init__(self, loop):
        self.loop = loop

    def start(self, *_args, **kwargs):
        if kwargs.get("update"):
            self.loop.update_now()
        self.pending = self.active = True

    def stop(self):
        self.active = self.pending = False

    def close(self):
        "Does nothing"


class Test(greentest.TestCase):

    def test_timeout_calls_update_now(self):
        hub = get_hub()
        loop = hub.loop
        proxy = _UpdateNowWithTimerProxy(loop)
        hub.loop = proxy

        try:
            with Timeout(DELAY * 2) as t:
                self.assertTrue(t.pending)
        finally:
            hub.loop = loop

        self.assertEqual(1, proxy.update_now_calls)

    def test_sleep_calls_update_now(self):
        hub = get_hub()
        loop = hub.loop
        proxy = _UpdateNowProxy(loop)
        hub.loop = proxy
        try:
            sleep(0.01)
        finally:
            hub.loop = loop

        self.assertEqual(1, proxy.update_now_calls)


    @greentest.skipOnAppVeyor("Timing is flaky, especially under Py 3.4/64-bit")
    @greentest.skipOnPyPy3OnCI("Timing is flaky, especially under Py 3.4/64-bit")
    @greentest.reraises_flaky_timeout((Timeout, AssertionError))
    def test_api(self):
        # Nothing happens if with-block finishes before the timeout expires
        t = Timeout(DELAY * 2)
        self.assertFalse(t.pending, t)
        with t:
            self.assertTrue(t.pending, t)
            sleep(DELAY)
        # check if timer was actually cancelled
        self.assertFalse(t.pending, t)
        sleep(DELAY * 2)

        # An exception will be raised if it's not
        with self.assertRaises(Timeout) as exc:
            with Timeout(DELAY) as t:
                sleep(DELAY * 10)

        self.assertIs(exc.exception, t)

        # You can customize the exception raised:
        with self.assertRaises(IOError):
            with Timeout(DELAY, IOError("Operation takes way too long")):
                sleep(DELAY * 10)

        # Providing classes instead of values should be possible too:
        with self.assertRaises(ValueError):
            with Timeout(DELAY, ValueError):
                sleep(DELAY * 10)


        try:
            1 / 0
        except ZeroDivisionError:
            with self.assertRaises(ZeroDivisionError):
                with Timeout(DELAY, sys.exc_info()[0]):
                    sleep(DELAY * 10)
                    raise AssertionError('should not get there')
                raise AssertionError('should not get there')
        else:
            raise AssertionError('should not get there')

        # It's possible to cancel the timer inside the block:
        with Timeout(DELAY) as timer:
            timer.cancel()
            sleep(DELAY * 2)

        # To silent the exception before exiting the block, pass False as second parameter.
        XDELAY = 0.1
        start = time.time()
        with Timeout(XDELAY, False):
            sleep(XDELAY * 2)
        delta = (time.time() - start)
        self.assertTimeWithinRange(delta, 0, XDELAY * 2)

        # passing None as seconds disables the timer
        with Timeout(None):
            sleep(DELAY)
        sleep(DELAY)

    def test_ref(self):
        err = Error()
        err_ref = weakref.ref(err)
        with Timeout(DELAY * 2, err):
            sleep(DELAY)
        del err
        gc.collect()
        self.assertFalse(err_ref(), err_ref)

    @flaky.reraises_flaky_race_condition()
    def test_nested_timeout(self):
        with Timeout(DELAY, False):
            with Timeout(DELAY * 10, False):
                sleep(DELAY * 3 * 20)
            raise AssertionError('should not get there')

        with Timeout(DELAY) as t1:
            with Timeout(DELAY * 20) as t2:
                with self.assertRaises(Timeout) as exc:
                    sleep(DELAY * 30)
                self.assertIs(exc.exception, t1)

                self.assertFalse(t1.pending, t1)
                self.assertTrue(t2.pending, t2)

            self.assertFalse(t2.pending)

        with Timeout(DELAY * 20) as t1:
            with Timeout(DELAY) as t2:
                with self.assertRaises(Timeout) as exc:
                    sleep(DELAY * 30)
                self.assertIs(exc.exception, t2)

                self.assertTrue(t1.pending, t1)
                self.assertFalse(t2.pending, t2)

        self.assertFalse(t1.pending)


if __name__ == '__main__':
    greentest.main()



"""
Cooperative implementation of special cases of :func:`signal.signal`.

This module is designed to work with libev's child watchers, as used
by default in :func:`gevent.os.fork` Note that each ``SIGCHLD``
handler will be run in a new greenlet when the signal is delivered
(just like :class:`gevent.hub.signal`)

The implementations in this module are only monkey patched if
:func:`gevent.os.waitpid` is being used (the default) and if
:const:`signal.SIGCHLD` is available; see :func:`gevent.os.fork` for
information on configuring this not to be the case for advanced uses.

.. versionadded:: 1.1b4
.. versionchanged:: 1.5a4
   Previously there was a backwards compatibility alias
   ``gevent.signal``, introduced in 1.1b4, that partly shadowed this
   module, confusing humans and static analysis tools alike. That alias
   has been removed. (See `gevent.signal_handler`.)
"""

from __future__ import absolute_import

from gevent._util import _NONE as _INITIAL
from gevent._util import copy_globals

import signal as _signal

__implements__ = []
__extensions__ = []


_child_handler = _INITIAL

_signal_signal = _signal.signal
_signal_getsignal = _signal.getsignal


def getsignal(signalnum):
    """
    Exactly the same as :func:`signal.getsignal` except where
    :const:`signal.SIGCHLD` is concerned.

    For :const:`signal.SIGCHLD`, this cooperates with :func:`signal`
    to provide consistent answers.
    """
    if signalnum != _signal.SIGCHLD:
        return _signal_getsignal(signalnum)

    global _child_handler
    if _child_handler is _INITIAL:
        _child_handler = _signal_getsignal(_signal.SIGCHLD)

    return _child_handler


def signal(signalnum, handler):
    """
    Exactly the same as :func:`signal.signal` except where
    :const:`signal.SIGCHLD` is concerned.

    .. note::

       A :const:`signal.SIGCHLD` handler installed with this function
       will only be triggered for children that are forked using
       :func:`gevent.os.fork` (:func:`gevent.os.fork_and_watch`);
       children forked before monkey patching, or otherwise by the raw
       :func:`os.fork`, will not trigger the handler installed by this
       function. (It's unlikely that a SIGCHLD handler installed with
       the builtin :func:`signal.signal` would be triggered either;
       libev typically overwrites such a handler at the C level. At
       the very least, it's full of race conditions.)

    .. note::

        Use of ``SIG_IGN`` and ``SIG_DFL`` may also have race conditions
        with libev child watchers and the :mod:`gevent.subprocess` module.

    .. versionchanged:: 1.2a1
         If ``SIG_IGN`` or ``SIG_DFL`` are used to ignore ``SIGCHLD``, a
         future use of ``gevent.subprocess`` and libev child watchers
         will once again work. However, on Python 2, use of ``os.popen``
         will fail.

    .. versionchanged:: 1.1rc2
         Allow using ``SIG_IGN`` and ``SIG_DFL`` to reset and ignore ``SIGCHLD``.
         However, this allows the possibility of a race condition if ``gevent.subprocess``
         had already been used.
    """
    if signalnum != _signal.SIGCHLD:
        return _signal_signal(signalnum, handler)

    # TODO: raise value error if not called from the main
    # greenlet, just like threads

    if handler != _signal.SIG_IGN and handler != _signal.SIG_DFL and not callable(handler):
        # exact same error message raised by the stdlib
        raise TypeError("signal handler must be signal.SIG_IGN, signal.SIG_DFL, or a callable object")

    old_handler = getsignal(signalnum)
    global _child_handler
    _child_handler = handler
    if handler in (_signal.SIG_IGN, _signal.SIG_DFL):
        # Allow resetting/ignoring this signal at the process level.
        # Note that this conflicts with gevent.subprocess and other users
        # of child watchers, until the next time gevent.subprocess/loop.install_sigchld()
        # is called.
        from gevent.hub import get_hub # Are we always safe to import here?
        _signal_signal(signalnum, handler)
        get_hub().loop.reset_sigchld()
    return old_handler


def _on_child_hook():
    # This is called in the hub greenlet. To let the function
    # do more useful work, like use blocking functions,
    # we run it in a new greenlet; see gevent.hub.signal
    if callable(_child_handler):
        # None is a valid value for the frame argument
        from gevent import Greenlet
        greenlet = Greenlet(_child_handler, _signal.SIGCHLD, None)
        greenlet.switch()


import gevent.os

if 'waitpid' in gevent.os.__implements__ and hasattr(_signal, 'SIGCHLD'):
    # Tightly coupled here to gevent.os and its waitpid implementation; only use these
    # if necessary.
    gevent.os._on_child_hook = _on_child_hook
    __implements__.append("signal")
    __implements__.append("getsignal")
else:
    # XXX: This breaks test__all__ on windows
    __extensions__.append("signal")
    __extensions__.append("getsignal")

__imports__ = copy_globals(_signal, globals(),
                           names_to_ignore=__implements__ + __extensions__,
                           dunder_names_to_keep=())

__all__ = __implements__ + __extensions__
